@@ -22,7 +22,7 @@ Z* -------------------------------------------------------------------
 #include"Setting.h"
 #include"os_gl.h"
 #include"Rep.h"
-
+#include"ObjectGadgetRamp.h"
 
 /* Compiled Graphics Library for simple graphics objects
    in floating point three-space, with the goal of achieving
@@ -31,12 +31,9 @@ Z* -------------------------------------------------------------------
 
 */
 
-#ifdef _PYMOL_CGO_DRAWARRAYS
 #define CGO_read_int(p) (*((int*)((p)++)))
-#else
-#define CGO_read_int(p) (*((int*)(p++)))
-#endif
 #define CGO_get_int(p) (*((int*)(p)))
+#define CGO_get_uint(p) (uint)(*((int*)(p)))
 #define CGO_write_int(p,i) ((*((int*)(p++)))=(i))
 #define CGO_put_int(p,i) ((*((int*)(p)))=(i))
 
@@ -58,31 +55,7 @@ Z* -------------------------------------------------------------------
  * primwidth * ray-tracing 
  */
 
-struct _CGO {
-  PyMOLGlobals *G;
-  float *op;
-  int c;
-  int z_flag;
-  float z_min, z_max;
-  float z_vector[3];
-  float alpha;
-  int *i_start, i_size;
-#ifdef _PYMOL_CGO_DRAWARRAYS
-  short has_begin_end;
-#endif
-  int current_pick_color_index, current_pick_color_bond;
-#ifdef _PYMOL_CGO_DRAWBUFFERS
-  float current_accessibility;
-  short has_draw_buffers, has_draw_cylinder_buffers, has_draw_sphere_buffers;
-  float normal[3], color[3], texture[2];
-  uchar pickColor[4];
-#endif
-  short use_shader, cgo_shader_ub_color, cgo_shader_ub_normal;
-  short debug;
-  short enable_shaders;
-  short no_pick;
-};
-
+#include "CGOStruct.h"
 
 /* instructions and data segment sizes */
 
@@ -146,7 +119,6 @@ struct _CGO {
 #define CGO_RESET_NORMAL_SZ      1
 #define CGO_PICK_COLOR           0x1F
 #define CGO_PICK_COLOR_SZ        2
-#ifdef _PYMOL_CGO_DRAWARRAYS
 /* CGO_DRAW_ARRAYS : operation that calls glDrawArrays with all arrays in memory 
    (i.e., stored in the CGO array). There can be up to 4 arrays (vertex, normal, color, 
    and pick color array, where each are stored using GL_FLOAT array (except for the pick  
@@ -162,8 +134,7 @@ struct _CGO {
 */
 #define CGO_DRAW_ARRAYS          0x1C
 #define CGO_DRAW_ARRAYS_SZ       0
-#endif
-#ifdef _PYMOL_CGO_DRAWBUFFERS
+#define CGO_DRAW_ARRAYS_HEADER       4
 /* CGO_DRAW_BUFFERS : operation that uses glDrawArrays with VBOs.  This is not currently
    used, since CGO_DRAW_BUFFERS_INDEXED is a bit more flexible where an index can be specified
    for vertex order.  However, this is useful for when all primitives are defined only once 
@@ -211,7 +182,6 @@ struct _CGO {
 #define CGO_DRAW_CYLINDER_BUFFERS       0x25
 #define CGO_DRAW_CYLINDER_BUFFERS_SZ    7
 
-#endif
 #define CGO_SHADER_CYLINDER             0x26
 #define CGO_SHADER_CYLINDER_SZ          8
 
@@ -268,14 +238,15 @@ int CGOGetExtent(CGO * I, float *mn, float *mx);
 int CGOHasNormals(CGO * I);
 
 void CGOFreeWithoutVBOs(CGO * I);
-void CGOFree(CGO * I);
+void CGOFree(CGO * &I);
 void CGOFreeImpl(CGO * I, short withVBOs);
 CGO *CGODrawText(CGO * I, int est, float *camera);
 
 CGO *CGOSimplify(CGO * I, int est);
 
+bool CGOCombineBeginEnd(CGO ** I, bool do_not_split_lines = false);
 CGO *CGOCombineBeginEnd(CGO * I, int est);
-#ifdef _PYMOL_CGO_DRAWBUFFERS
+
 void CGOFreeVBOs(CGO *I);
 CGO *CGOOptimizeToVBOIndexedWithColor(CGO * I, int est, float *color);
 CGO *CGOOptimizeToVBOIndexedNoShader(CGO * I, int est);
@@ -283,8 +254,9 @@ CGO *CGOOptimizeToVBOIndexed(CGO * I, int est);
 CGO *CGOOptimizeToVBONotIndexedWithReturnedData(CGO * I, int est, short, float **);
 CGO *CGOOptimizeToVBONotIndexed(CGO * I, int est);
 CGO *CGOOptimizeSpheresToVBONonIndexedImpl(CGO * I, int est, CGO *leftOverCGO);
-CGO *CGOOptimizeSpheresToVBONonIndexed(CGO * I, int est);
-#endif
+
+CGO *CGOOptimizeSpheresToVBONonIndexed(CGO * I, int est=0, bool addshaders=false, CGO *leftOverCGO=NULL);
+#define CGOOptimizeSpheresToVBONonIndexedNoShader(I, est) CGOOptimizeSpheresToVBONonIndexed(I, est, false, NULL)
 
 void CGOReserve(CGO * ptr, int est);
 
@@ -293,24 +265,24 @@ int CGOPreloadFonts(CGO * I);
 
 int CGOCheckForText(CGO * I);
 
-int CGOFromFloatArray(CGO * I, float *src, int len);
+int CGOFromFloatArray(CGO * I, const float *src, int len);
 
 int CGOBegin(CGO * I, int mode);
 int CGOEnd(CGO * I);
 
-int CGOSphere(CGO * I, float *v1, float r);
-int CGOEllipsoid(CGO * I, float *v1, float r, float *n1, float *n2, float *n3);
-int CGOQuadric(CGO * I, float *v1, float r, float *p); /* NOT WORKING YET */
-int CGOSausage(CGO * I, float *v1, float *v2, float r, float *c1, float *c2);
+int CGOSphere(CGO * I, const float *v1, float r);
+int CGOEllipsoid(CGO * I, const float *v1, float r, const float *n1, const float *n2, const float *n3);
+int CGOQuadric(CGO * I, const float *v1, float r, const float *p); /* NOT WORKING YET */
+int CGOSausage(CGO * I, const float *v1, const float *v2, float r, const float *c1, const float *c2);
 int CGOVertex(CGO * I, float v1, float v2, float v3);
-int CGOVertexv(CGO * I, float *v);
+int CGOVertexv(CGO * I, const float *v);
 int CGOAlpha(CGO * I, float alpha);
 int CGOColor(CGO * I, float v1, float v2, float v3);
-int CGOColorv(CGO * I, float *v);
+int CGOColorv(CGO * I, const float *v);
 int CGOTexCoord2f(CGO * I, float v1, float v2);
-int CGOTexCoord2fv(CGO * I, float *v);
+int CGOTexCoord2fv(CGO * I, const float *v);
 int CGONormal(CGO * I, float v1, float v2, float v3);
-int CGONormalv(CGO * I, float *v);
+int CGONormalv(CGO * I, const float *v);
 int CGOResetNormal(CGO * I, int mode);
 int CGOLinewidth(CGO * I, float v);
 int CGOLinewidthSpecial(CGO * I, int v);
@@ -320,24 +292,24 @@ int CGOLinewidthSpecial(CGO * I, int v);
 #define LINEWIDTH_DYNAMIC_WITH_SCALE_RIBBON 4
 #define LINEWIDTH_DYNAMIC_WITH_SCALE_DASH 5
 #define CYLINDERWIDTH_DYNAMIC_MESH  6
+#define LINEWIDTH_WITH_SCALE 7
 int CGODotwidth(CGO * I, float v);
 int CGOChar(CGO * I, char c);
 int CGOFontVertex(CGO * I, float x, float y, float z);
-int CGOFontVertexv(CGO * I, float *v);
+int CGOFontVertexv(CGO * I, const float *v);
 int CGOFontScale(CGO * I, float v1, float v2);
 int CGOIndent(CGO * I, char c, float dir);
-int CGOWrite(CGO * I, char *str);
-int CGOWriteLeft(CGO * I, char *str);
-int CGOWriteIndent(CGO * I, char *str, float indent);
+int CGOWrite(CGO * I, const char *str);
+int CGOWriteLeft(CGO * I, const char *str);
+int CGOWriteIndent(CGO * I, const char *str, float indent);
 
 GLfloat *CGODrawArrays(CGO *I, GLenum mode, short arrays, int nverts);
 
-#ifdef _PYMOL_CGO_DRAWBUFFERS
 int CGODrawBuffers(CGO *I, GLenum mode, short arrays, int nverts, uint *bufs);
 GLfloat *CGODrawBuffersIndexed(CGO *I, GLenum mode, short arrays, int nindices, int nverts, uint *bufs);
 int CGOBoundingBox(CGO *I, float *min, float *max);
 int CGOAccessibility(CGO * I, float a);
-#endif
+
 int CGODrawTexture(CGO *I, int texture_id, float *worldPos, float *screenMin, float *screenMax, float *textExtent);
 int CGODrawLabel(CGO *I, int texture_id, float *worldPos, float *screenWorldOffset, float *screenMin, float *screenMax, float *textExtent);
 CGO *CGOOptimizeLabels(CGO * I, int est);
@@ -352,16 +324,16 @@ int CGODisable(CGO * I, int mode);
 
 int CGOStop(CGO * I);
 
-int CGOCylinderv(CGO * I, float *p1, float *p2, float r, float *c1, float *c2);
-int CGOCustomCylinderv(CGO * I, float *p1, float *p2, float r, float *c1, float *c2,
+int CGOCylinderv(CGO * I, const float *p1, const float *p2, float r, const float *c1, const float *c2);
+int CGOCustomCylinderv(CGO * I, const float *p1, const float *p2, float r, const float *c1, const float *c2,
                         float cap1, float cap2);
-int CGOConev(CGO * I, float *p1, float *p2, float r1, float r2, float *c1, float *c2,
+int CGOConev(CGO * I, const float *p1, const float *p2, float r1, float r2, const float *c1, const float *c2,
               float cap1, float cap2);
 
 int CGOAlphaTriangle(CGO * I,
-		     float *v1, float *v2, float *v3,
-		     float *n1, float *n2, float *n3,
-		     float *c1, float *c2, float *c3,
+		     const float *v1, const float *v2, const float *v3,
+		     const float *n1, const float *n2, const float *n3,
+		     const float *c1, const float *c2, const float *c3,
 		     float a1, float a2, float a3, int reverse);
 void CGOSetZVector(CGO * I, float z0, float z1, float z2);
 void CGORenderGLPicking(CGO * I, Picking ** pick,
@@ -392,14 +364,15 @@ CGO *CGOOptimizeGLSLCylindersToVBOIndexed(CGO * I, int est);
 CGO *CGOOptimizeGLSLCylindersToVBOIndexedWithLeftOver(CGO * I, int est, CGO *leftOverCGO);
 CGO *CGOOptimizeGLSLCylindersToVBOIndexedNoColor(CGO * I, int est);
 
-int CGOShaderCylinder(CGO *I, float *origin, float *axis, float tube_size, int cap);
-int CGOShaderCylinder2ndColor(CGO *I, float *origin, float *axis, float tube_size, int cap, float *color2);
+int CGOShaderCylinder(CGO *I, const float *origin, const float *axis, float tube_size, int cap);
+int CGOShaderCylinder2ndColor(CGO *I, const float *origin, const float *axis, float tube_size, int cap, const float *color2);
 
 int CGOCountNumberOfOperationsOfTypeDEBUG(CGO *I, int optype);
 int CGOCountNumberOfOperationsOfType(CGO *I, int op);
-short CGOHasOperationsOfType(CGO *I, int op);
+short CGOHasOperationsOfType(const CGO *I, int op);
 short CGOHasOperationsOfType2(CGO *I, int op1, int op2);
 short CGOHasCylinderOperations(CGO *I);
+bool CGOHasSphereOperations(const CGO *I);
 
 short CGOCheckWhetherToFree(PyMOLGlobals * G, CGO *I);
 
